@@ -97,15 +97,11 @@ void print_version(void)
 time_t get_localtime(struct tm *rectime, int d_off)
 {
 	time_t timer;
-	struct tm *ltm;
 
 	time(&timer);
 	timer -= SEC_PER_DAY * d_off;
-	ltm = localtime(&timer);
+	localtime_r(&timer, rectime);
 
-	if (ltm) {
-		*rectime = *ltm;
-	}
 	return timer;
 }
 
@@ -126,15 +122,11 @@ time_t get_localtime(struct tm *rectime, int d_off)
 time_t get_gmtime(struct tm *rectime, int d_off)
 {
 	time_t timer;
-	struct tm *ltm;
 
 	time(&timer);
 	timer -= SEC_PER_DAY * d_off;
-	ltm = gmtime(&timer);
+	gmtime_r(&timer, rectime);
 
-	if (ltm) {
-		*rectime = *ltm;
-	}
 	return timer;
 }
 
@@ -776,9 +768,9 @@ char *strtolower(char *str)
 */
 char *get_persistent_type_dir(char *type)
 {
-	static char dir[PATH_MAX];
+	static char dir[32];
 
-	snprintf(dir, sizeof(dir), "%s-%s", DEV_DISK_BY, type);
+	snprintf(dir, 32, "%s-%s", DEV_DISK_BY, type);
 
 	if (access(dir, R_OK)) {
 		return (NULL);
