@@ -1,6 +1,6 @@
 /*
  * sysstat: System performance tools for Linux
- * (C) 1999-2017 by Sebastien Godard (sysstat <at> orange.fr)
+ * (C) 1999-2018 by Sebastien Godard (sysstat <at> orange.fr)
  */
 
 #ifndef _COMMON_H
@@ -12,14 +12,13 @@
 #include <time.h>
 #include <sched.h>	/* For __CPU_SETSIZE */
 #include <limits.h>
+#include <stdlib.h>
 
 #ifdef HAVE_SYS_SYSMACROS_H
 /* Needed on some non-glibc environments */
 #include <sys/sysmacros.h>
 #endif
 
-#include "rd_stats.h"
-
 /*
  ***************************************************************************
  * Various keywords and constants
@@ -55,7 +54,7 @@
 #endif
 
 /* Maximum number of interrupts */
-#define NR_IRQS		1024
+#define NR_IRQS		4096
 
 /* Size of /proc/interrupts line, CPU data excluded */
 #define INTERRUPTS_LINE	128
@@ -89,7 +88,7 @@
 #define SYSFS_PRODUCT		"product"
 #define SYSFS_FCHOST		"/sys/class/fc_host"
 
-#define MAX_FILE_LEN		256
+#define MAX_FILE_LEN		512
 #define MAX_PF_NAME		1024
 #define MAX_NAME_LEN		128
 
@@ -123,7 +122,7 @@
 /* Allocate and init structure */
 #define SREALLOC(S, TYPE, SIZE)	do {								 \
 					TYPE *_p_ = S;						 \
-   				   	if (SIZE) {						 \
+					if ((SIZE) != 0) {					 \
 						if ((S = (TYPE *) realloc(S, (SIZE))) == NULL) { \
 				         		perror("realloc");			 \
 				         		exit(4);				 \
@@ -143,10 +142,10 @@
 /*
  * Macros used to display statistics values.
  *
- * NB: Define SP_VALUE() to normalize to %;
- * HZ is 1024 on IA64 and % should be normalized to 100.
  */
-#define S_VALUE(m,n,p)		(((double) ((n) - (m))) / (p) * HZ)
+/* With S_VALUE macro, the interval of time (@p) is given in 1/100th of a second */
+#define S_VALUE(m,n,p)		(((double) ((n) - (m))) / (p) * 100)
+/* Define SP_VALUE() to normalize to % */
 #define SP_VALUE(m,n,p)		(((double) ((n) - (m))) / (p) * 100)
 
 /*
@@ -166,7 +165,7 @@
 
 /* Number of ticks per second */
 #define HZ		hz
-extern unsigned int hz;
+extern unsigned long hz;
 
 /* Number of bit shifts to convert pages to kB */
 extern unsigned int kb_shift;
@@ -217,7 +216,6 @@ extern char persistent_name_type[MAX_FILE_LEN];
 struct ext_disk_stats {
 	double util;
 	double await;
-	double svctm;
 	double arqsz;
 };
 
@@ -226,10 +224,24 @@ struct ext_disk_stats {
  * Functions prototypes
  ***************************************************************************
  */
+void print_version
+	(void);
+void get_HZ
+	(void);
+void get_kb_shift
+	(void);
+time_t get_localtime
+	(struct tm *, int);
+time_t get_time
+	(struct tm *, int);
+void init_nls
+	(void);
+int is_device
+	(char *, int);
+void sysstat_panic
+	(const char *, int);
 
-void compute_ext_disk_stats
-	(struct stats_disk *, struct stats_disk *, unsigned long long,
-	 struct ext_disk_stats *);
+#ifndef SOURCE_SADC
 int count_bits
 	(void *, int);
 int count_csvalues
@@ -248,20 +260,10 @@ void cprintf_x
 	(int, int, ...);
 char *device_name
 	(char *);
-void get_HZ
-	(void);
 unsigned int get_devmap_major
 	(void);
 unsigned long long get_interval
 	(unsigned long long, unsigned long long);
-void get_kb_shift
-	(void);
-time_t get_localtime
-	(struct tm *, int);
-time_t get_time
-	(struct tm *, int);
-unsigned long long get_per_cpu_interval
-	(struct stats_cpu *, struct stats_cpu *);
 char *get_persistent_name_from_pretty
 	(char *);
 char *get_persistent_type_dir
@@ -274,10 +276,6 @@ int get_win_height
 	(void);
 void init_colors
 	(void);
-void init_nls
-	(void);
-int is_device
-	(char *, int);
 double ll_sp_value
 	(unsigned long long, unsigned long long, unsigned long long);
 int is_iso_time_fmt
@@ -286,17 +284,14 @@ int parse_values
 	(char *, unsigned char[], int, const char *);
 int print_gal_header
 	(struct tm *, char *, char *, char *, char *, int, int);
-void print_version
-	(void);
 int set_report_date
 	(struct tm *, char[], int);
 char *strtolower
 	(char *);
-void sysstat_panic
-	(const char *, int);
 void xprintf
 	(int, const char *, ...);
 void xprintf0
 	(int, const char *, ...);
 
+#endif /* SOURCE_SADC undefined */
 #endif  /* _COMMON_H */
