@@ -77,6 +77,8 @@ struct io_stats {
 	unsigned long rd_sectors	__attribute__ ((aligned (8)));
 	/* # of sectors written */
 	unsigned long wr_sectors	__attribute__ ((packed));
+	/* # of sectors discarded */
+	unsigned long dc_sectors	__attribute__ ((packed));
 	/* # of read operations issued to the device */
 	unsigned long rd_ios		__attribute__ ((packed));
 	/* # of read requests merged */
@@ -85,10 +87,16 @@ struct io_stats {
 	unsigned long wr_ios		__attribute__ ((packed));
 	/* # of write requests merged */
 	unsigned long wr_merges		__attribute__ ((packed));
+	/* # of discard operations issued to the device */
+	unsigned long dc_ios		__attribute__ ((packed));
+	/* # of discard requests merged */
+	unsigned long dc_merges		__attribute__ ((packed));
 	/* Time of read requests in queue */
 	unsigned int  rd_ticks		__attribute__ ((packed));
 	/* Time of write requests in queue */
 	unsigned int  wr_ticks		__attribute__ ((packed));
+	/* Time of discard requests in queue */
+	unsigned int  dc_ticks		__attribute__ ((packed));
 	/* # of I/Os in progress */
 	unsigned int  ios_pgr		__attribute__ ((packed));
 	/* # of ticks total (for this device) for I/O */
@@ -104,20 +112,28 @@ struct ext_io_stats {
 	double r_await;
 	/* w_await */
 	double w_await;
+	/* d_await */
+	double d_await;
 	/* rsec/s */
 	double rsectors;
 	/* wsec/s */
 	double wsectors;
+	/* dsec/s */
+	double dsectors;
 	/* sec/s */
 	double sectors;
 	/* %rrqm */
 	double rrqm_pc;
 	/* %wrqm */
 	double wrqm_pc;
+	/* %drqm */
+	double drqm_pc;
 	/* rareq-sz */
 	double rarqsz;
 	/* wareq-sz */
 	double warqsz;
+	/* dareq-sz */
+	double darqsz;
 };
 
 /* Possible values for field "status" in io_hdr_stats structure */
