@@ -218,8 +218,8 @@ __print_funct_t print_cpu_stats(struct activity *a, int prev, int curr,
 				 * %irq, %soft, %guest, %gnice.
 				 */
 				else if (DISPLAY_CPU_ALL(a->opt_flags)) {
-					cprintf_pc(DISPLAY_UNIT(flags), 5, 9, 2,
-						   0.0, 0.0, 0.0, 0.0, 100.0);
+					cprintf_pc(DISPLAY_UNIT(flags), 4, 9, 2,
+						   0.0, 0.0, 0.0, 100.0);
 					printf("\n");
 				}
 				continue;
@@ -454,17 +454,21 @@ __print_funct_t print_io_stats(struct activity *a, int prev, int curr,
 	 * We display 0.0 in this case though we should rather tell
 	 * the user that the value cannot be calculated here.
 	 */
-	cprintf_f(NO_UNIT, 5, 9, 2,
+	cprintf_f(NO_UNIT, 7, 9, 2,
 		  sic->dk_drive < sip->dk_drive ? 0.0 :
 		  S_VALUE(sip->dk_drive, sic->dk_drive, itv),
 		  sic->dk_drive_rio < sip->dk_drive_rio ? 0.0 :
 		  S_VALUE(sip->dk_drive_rio, sic->dk_drive_rio, itv),
 		  sic->dk_drive_wio < sip->dk_drive_wio ? 0.0 :
 		  S_VALUE(sip->dk_drive_wio, sic->dk_drive_wio, itv),
+		  sic->dk_drive_dio < sip->dk_drive_dio ? 0.0 :
+		  S_VALUE(sip->dk_drive_dio, sic->dk_drive_dio, itv),
 		  sic->dk_drive_rblk < sip->dk_drive_rblk ? 0.0 :
 		  S_VALUE(sip->dk_drive_rblk, sic->dk_drive_rblk, itv),
 		  sic->dk_drive_wblk < sip->dk_drive_wblk ? 0.0 :
-		  S_VALUE(sip->dk_drive_wblk, sic->dk_drive_wblk, itv));
+		  S_VALUE(sip->dk_drive_wblk, sic->dk_drive_wblk, itv),
+		  sic->dk_drive_dblk < sip->dk_drive_dblk ? 0.0 :
+		  S_VALUE(sip->dk_drive_dblk, sic->dk_drive_dblk, itv));
 	printf("\n");
 }
 
@@ -1068,16 +1072,16 @@ __print_funct_t print_disk_stats(struct activity *a, int prev, int curr,
 		}
 		cprintf_f(NO_UNIT, 1, 9, 2,
 			  S_VALUE(sdp->nr_ios, sdc->nr_ios,  itv));
-		cprintf_f(unit, 2, 9, 2,
+		cprintf_f(unit, 3, 9, 2,
 			  S_VALUE(sdp->rd_sect, sdc->rd_sect, itv) / 2,
-			  S_VALUE(sdp->wr_sect, sdc->wr_sect, itv) / 2);
+			  S_VALUE(sdp->wr_sect, sdc->wr_sect, itv) / 2,
+			  S_VALUE(sdp->dc_sect, sdc->dc_sect, itv) / 2);
 		/* See iostat for explanations */
 		cprintf_f(unit, 1, 9, 2,
 			  xds.arqsz / 2);
-		cprintf_f(NO_UNIT, 3, 9, 2,
+		cprintf_f(NO_UNIT, 2, 9, 2,
 			  S_VALUE(sdp->rq_ticks, sdc->rq_ticks, itv) / 1000.0,
-			  xds.await,
-			  xds.svctm);
+			  xds.await);
 		cprintf_pc(DISPLAY_UNIT(flags), 1, 9, 2,
 			   xds.util / 10.0);
 		if (DISPLAY_HUMAN_READ(flags)) {
@@ -2427,7 +2431,9 @@ void stub_print_huge_stats(struct activity *a, int curr, int dispavg)
 		*smc = (struct stats_huge *) a->buf[curr];
 	static unsigned long long
 		avg_frhkb = 0,
-		avg_tlhkb = 0;
+		avg_tlhkb = 0,
+		avg_rsvdhkb = 0,
+		avg_surphkb = 0;
 	int unit = NO_UNIT;
 
 	if (DISPLAY_UNIT(flags)) {
@@ -2448,11 +2454,16 @@ void stub_print_huge_stats(struct activity *a, int curr, int dispavg)
 		cprintf_pc(DISPLAY_UNIT(flags), 1, 9, 2,
 			   smc->tlhkb ?
 			   SP_VALUE(smc->frhkb, smc->tlhkb, smc->tlhkb) : 0.0);
+		cprintf_u64(unit, 2, 9,
+			    (unsigned long long) smc->rsvdhkb,
+			    (unsigned long long) (smc->surphkb));
 		printf("\n");
 
 		/* Will be used to compute the average */
 		avg_frhkb += smc->frhkb;
 		avg_tlhkb += smc->tlhkb;
+		avg_rsvdhkb += smc->rsvdhkb;
+		avg_surphkb += smc->surphkb;
 	}
 	else {
 		/* Display average values */
@@ -2466,10 +2477,13 @@ void stub_print_huge_stats(struct activity *a, int curr, int dispavg)
 			   SP_VALUE((double) avg_frhkb / avg_count,
 				    (double) avg_tlhkb / avg_count,
 				    (double) avg_tlhkb / avg_count) : 0.0);
+		cprintf_f(unit, 2, 9, 0,
+			  (double) avg_rsvdhkb / avg_count,
+			  (double) avg_surphkb / avg_count);
 		printf("\n");
 
 		/* Reset average counters */
-		avg_frhkb = avg_tlhkb = 0;
+		avg_frhkb = avg_tlhkb = avg_rsvdhkb = avg_surphkb = 0;
 	}
 }
 
