@@ -1,12 +1,11 @@
 /*
  * rd_stats.h: Include file used to read system statistics
- * (C) 1999-2017 by Sebastien Godard (sysstat <at> orange.fr)
+ * (C) 1999-2018 by Sebastien Godard (sysstat <at> orange.fr)
  */
 
 #ifndef _RD_STATS_H
 #define _RD_STATS_H
 
-
 /*
  ***************************************************************************
  * Miscellaneous constants
@@ -19,6 +18,8 @@
 #define IFNAMSIZ	16
 #endif
 
+/* Maximum length of block device name */
+#define MAX_DEV_LEN	128
 /* Maximum length of network interface name */
 #define MAX_IFACE_LEN	IFNAMSIZ
 /* Maximum length of USB manufacturer string */
@@ -40,6 +41,9 @@
 #define C_DUPLEX_HALF	1
 #define C_DUPLEX_FULL	2
 
+/* Type for all functions counting items. Value can be negative (-1) */
+#define __nr_t		int
+
 /*
  ***************************************************************************
  * System files containing statistics
@@ -74,13 +78,19 @@
 
 /*
  ***************************************************************************
- * Definitions of structures for system statistics
+ * Definitions of structures for system statistics.
+ * WARNING: Fields order matters for SVG graphs!
  ***************************************************************************
  */
 
-#define ULL_ALIGNMENT_WIDTH 16
-#define UL_ALIGNMENT_WIDTH 8
-#define U_ALIGNMENT_WIDTH 4
+#define SIZEOF_LONG_64BIT	8
+#define ULL_ALIGNMENT_WIDTH	8
+#define UL_ALIGNMENT_WIDTH	SIZEOF_LONG_64BIT
+#define U_ALIGNMENT_WIDTH	4
+
+#define MAP_SIZE(m)	((m[0] * ULL_ALIGNMENT_WIDTH) + \
+			 (m[1] * UL_ALIGNMENT_WIDTH) +  \
+			 (m[2] * U_ALIGNMENT_WIDTH))
 
 /*
  * Structure for CPU statistics.
@@ -88,19 +98,22 @@
  * Following structures are for each individual CPU (0, 1, etc.)
  */
 struct stats_cpu {
-	unsigned long long cpu_user		__attribute__ ((aligned (16)));
-	unsigned long long cpu_nice		__attribute__ ((aligned (16)));
-	unsigned long long cpu_sys		__attribute__ ((aligned (16)));
-	unsigned long long cpu_idle		__attribute__ ((aligned (16)));
-	unsigned long long cpu_iowait		__attribute__ ((aligned (16)));
-	unsigned long long cpu_steal		__attribute__ ((aligned (16)));
-	unsigned long long cpu_hardirq		__attribute__ ((aligned (16)));
-	unsigned long long cpu_softirq		__attribute__ ((aligned (16)));
-	unsigned long long cpu_guest		__attribute__ ((aligned (16)));
-	unsigned long long cpu_guest_nice	__attribute__ ((aligned (16)));
+	unsigned long long cpu_user;
+	unsigned long long cpu_nice;
+	unsigned long long cpu_sys;
+	unsigned long long cpu_idle;
+	unsigned long long cpu_iowait;
+	unsigned long long cpu_steal;
+	unsigned long long cpu_hardirq;
+	unsigned long long cpu_softirq;
+	unsigned long long cpu_guest;
+	unsigned long long cpu_guest_nice;
 };
 
 #define STATS_CPU_SIZE	(sizeof(struct stats_cpu))
+#define STATS_CPU_ULL	10
+#define STATS_CPU_UL	0
+#define STATS_CPU_U	0
 
 /*
  * Structure for task creation and context switch statistics.
@@ -108,25 +121,28 @@ struct stats_cpu {
  * the same value on 32 and 64-bit architectures.
  */
 struct stats_pcsw {
-	unsigned long long context_switch	__attribute__ ((aligned (16)));
-	unsigned long processes			__attribute__ ((aligned (16)));
+	unsigned long long context_switch;
+	unsigned long	   processes	__attribute__ ((aligned (8)));
 };
 
 #define STATS_PCSW_SIZE	(sizeof(struct stats_pcsw))
+#define STATS_PCSW_ULL	1
+#define STATS_PCSW_UL	1
+#define STATS_PCSW_U	0
 
 /*
  * Structure for interrupts statistics.
  * In activity buffer: First structure is for total number of interrupts ("SUM").
  * Following structures are for each individual interrupt (0, 1, etc.)
- *
- * NOTE: The total number of interrupts is saved as a %llu by the kernel,
- * whereas individual interrupts are saved as %u.
  */
 struct stats_irq {
-	unsigned long long irq_nr	__attribute__ ((aligned (16)));
+	unsigned long long irq_nr;
 };
 
 #define STATS_IRQ_SIZE	(sizeof(struct stats_irq))
+#define STATS_IRQ_ULL	1
+#define STATS_IRQ_UL	0
+#define STATS_IRQ_U	0
 
 /* Structure for swapping statistics */
 struct stats_swap {
@@ -135,6 +151,9 @@ struct stats_swap {
 };
 
 #define STATS_SWAP_SIZE	(sizeof(struct stats_swap))
+#define STATS_SWAP_ULL	0
+#define STATS_SWAP_UL	2
+#define STATS_SWAP_U	0
 
 /* Structure for paging statistics */
 struct stats_paging {
@@ -149,195 +168,239 @@ struct stats_paging {
 };
 
 #define STATS_PAGING_SIZE	(sizeof(struct stats_paging))
+#define STATS_PAGING_ULL	0
+#define STATS_PAGING_UL		8
+#define STATS_PAGING_U		0
 
 /* Structure for I/O and transfer rate statistics */
 struct stats_io {
-	unsigned long long dk_drive		__attribute__ ((aligned (16)));
-	unsigned long long dk_drive_rio		__attribute__ ((packed));
-	unsigned long long dk_drive_wio		__attribute__ ((packed));
-	unsigned long long dk_drive_rblk	__attribute__ ((packed));
-	unsigned long long dk_drive_wblk	__attribute__ ((packed));
+	unsigned long long dk_drive;
+	unsigned long long dk_drive_rio;
+	unsigned long long dk_drive_wio;
+	unsigned long long dk_drive_rblk;
+	unsigned long long dk_drive_wblk;
+	unsigned long long dk_drive_dio;
+	unsigned long long dk_drive_dblk;
 };
 
 #define STATS_IO_SIZE	(sizeof(struct stats_io))
+#define STATS_IO_ULL	7
+#define STATS_IO_UL	0
+#define STATS_IO_U	0
 
 /* Structure for memory and swap space utilization statistics */
 struct stats_memory {
-	unsigned long frmkb		__attribute__ ((aligned (8)));
-	unsigned long bufkb		__attribute__ ((aligned (8)));
-	unsigned long camkb		__attribute__ ((aligned (8)));
-	unsigned long tlmkb		__attribute__ ((aligned (8)));
-	unsigned long frskb		__attribute__ ((aligned (8)));
-	unsigned long tlskb		__attribute__ ((aligned (8)));
-	unsigned long caskb		__attribute__ ((aligned (8)));
-	unsigned long comkb		__attribute__ ((aligned (8)));
-	unsigned long activekb		__attribute__ ((aligned (8)));
-	unsigned long inactkb		__attribute__ ((aligned (8)));
-	unsigned long dirtykb		__attribute__ ((aligned (8)));
-	unsigned long anonpgkb		__attribute__ ((aligned (8)));
-	unsigned long slabkb		__attribute__ ((aligned (8)));
-	unsigned long kstackkb		__attribute__ ((aligned (8)));
-	unsigned long pgtblkb		__attribute__ ((aligned (8)));
-	unsigned long vmusedkb		__attribute__ ((aligned (8)));
-	unsigned long availablekb	__attribute__ ((aligned (8)));
+	unsigned long long frmkb;
+	unsigned long long bufkb;
+	unsigned long long camkb;
+	unsigned long long tlmkb;
+	unsigned long long frskb;
+	unsigned long long tlskb;
+	unsigned long long caskb;
+	unsigned long long comkb;
+	unsigned long long activekb;
+	unsigned long long inactkb;
+	unsigned long long dirtykb;
+	unsigned long long anonpgkb;
+	unsigned long long slabkb;
+	unsigned long long kstackkb;
+	unsigned long long pgtblkb;
+	unsigned long long vmusedkb;
+	unsigned long long availablekb;
 };
 
 #define STATS_MEMORY_SIZE	(sizeof(struct stats_memory))
+#define STATS_MEMORY_ULL	17
+#define STATS_MEMORY_UL		0
+#define STATS_MEMORY_U		0
 
 /* Structure for kernel tables statistics */
 struct stats_ktables {
-	unsigned int  file_used		__attribute__ ((aligned (4)));
-	unsigned int  inode_used	__attribute__ ((packed));
-	unsigned int  dentry_stat	__attribute__ ((packed));
-	unsigned int  pty_nr		__attribute__ ((packed));
+	unsigned long long file_used;
+	unsigned long long inode_used;
+	unsigned long long dentry_stat;
+	unsigned long long pty_nr;
 };
 
 #define STATS_KTABLES_SIZE	(sizeof(struct stats_ktables))
+#define STATS_KTABLES_ULL	4
+#define STATS_KTABLES_UL	0
+#define STATS_KTABLES_U		0
 
 /* Structure for queue and load statistics */
 struct stats_queue {
-	unsigned long nr_running	__attribute__ ((aligned (8)));
-	unsigned long procs_blocked	__attribute__ ((aligned (8)));
-	unsigned int  load_avg_1	__attribute__ ((aligned (8)));
-	unsigned int  load_avg_5	__attribute__ ((packed));
-	unsigned int  load_avg_15	__attribute__ ((packed));
-	unsigned int  nr_threads	__attribute__ ((packed));
+	unsigned long long nr_running;
+	unsigned long long procs_blocked;
+	unsigned long long nr_threads;
+	unsigned int	   load_avg_1;
+	unsigned int	   load_avg_5;
+	unsigned int	   load_avg_15;
 };
 
 #define STATS_QUEUE_SIZE	(sizeof(struct stats_queue))
+#define STATS_QUEUE_ULL		3
+#define STATS_QUEUE_UL		0
+#define STATS_QUEUE_U		3
 
 /* Structure for serial statistics */
 struct stats_serial {
-	unsigned int rx		__attribute__ ((aligned (4)));
-	unsigned int tx		__attribute__ ((packed));
-	unsigned int frame	__attribute__ ((packed));
-	unsigned int parity	__attribute__ ((packed));
-	unsigned int brk	__attribute__ ((packed));
-	unsigned int overrun	__attribute__ ((packed));
-	/*
-	 * A value of 0 means that the structure is unused.
-	 * To avoid the confusion, the line number is saved as (line# + 1)
-	 */
-	unsigned int line	__attribute__ ((packed));
+	unsigned int rx;
+	unsigned int tx;
+	unsigned int frame;
+	unsigned int parity;
+	unsigned int brk;
+	unsigned int overrun;
+	unsigned int line;
 };
 
 #define STATS_SERIAL_SIZE	(sizeof(struct stats_serial))
+#define STATS_SERIAL_ULL	0
+#define STATS_SERIAL_UL		0
+#define STATS_SERIAL_U		7
 
 /* Structure for block devices statistics */
 struct stats_disk {
-	unsigned long long nr_ios	__attribute__ ((aligned (16)));
-	unsigned long rd_sect		__attribute__ ((aligned (16)));
-	unsigned long wr_sect		__attribute__ ((aligned (8)));
-	unsigned int rd_ticks		__attribute__ ((aligned (8)));
-	unsigned int wr_ticks		__attribute__ ((packed));
-	unsigned int tot_ticks		__attribute__ ((packed));
-	unsigned int rq_ticks		__attribute__ ((packed));
-	unsigned int major		__attribute__ ((packed));
-	unsigned int minor		__attribute__ ((packed));
+	unsigned long long nr_ios;
+	unsigned long	   rd_sect	__attribute__ ((aligned (8)));
+	unsigned long	   wr_sect	__attribute__ ((aligned (8)));
+	unsigned long	   dc_sect	__attribute__ ((aligned (8)));
+	unsigned int	   rd_ticks	__attribute__ ((aligned (8)));
+	unsigned int	   wr_ticks;
+	unsigned int	   tot_ticks;
+	unsigned int	   rq_ticks;
+	unsigned int	   major;
+	unsigned int	   minor;
+	unsigned int	   dc_ticks;
 };
 
 #define STATS_DISK_SIZE	(sizeof(struct stats_disk))
+#define STATS_DISK_ULL	1
+#define STATS_DISK_UL	3
+#define STATS_DISK_U	7
 
 /* Structure for network interfaces statistics */
 struct stats_net_dev {
-	unsigned long long rx_packets		__attribute__ ((aligned (16)));
-	unsigned long long tx_packets		__attribute__ ((aligned (16)));
-	unsigned long long rx_bytes		__attribute__ ((aligned (16)));
-	unsigned long long tx_bytes		__attribute__ ((aligned (16)));
-	unsigned long long rx_compressed	__attribute__ ((aligned (16)));
-	unsigned long long tx_compressed	__attribute__ ((aligned (16)));
-	unsigned long long multicast		__attribute__ ((aligned (16)));
-	unsigned int       speed		__attribute__ ((aligned (16)));
-	char 	 interface[MAX_IFACE_LEN]	__attribute__ ((aligned (4)));
-	char	 duplex;
+	unsigned long long rx_packets;
+	unsigned long long tx_packets;
+	unsigned long long rx_bytes;
+	unsigned long long tx_bytes;
+	unsigned long long rx_compressed;
+	unsigned long long tx_compressed;
+	unsigned long long multicast;
+	unsigned int	   speed;
+	char		   interface[MAX_IFACE_LEN];
+	char		   duplex;
 };
 
 #define STATS_NET_DEV_SIZE	(sizeof(struct stats_net_dev))
+#define STATS_NET_DEV_SIZE2CMP	(STATS_NET_DEV_SIZE - MAX_IFACE_LEN - 1)
+#define STATS_NET_DEV_ULL	7
+#define STATS_NET_DEV_UL	0
+#define STATS_NET_DEV_U		1
 
 /* Structure for network interface errors statistics */
 struct stats_net_edev {
-	unsigned long long collisions		__attribute__ ((aligned (16)));
-	unsigned long long rx_errors		__attribute__ ((aligned (16)));
-	unsigned long long tx_errors		__attribute__ ((aligned (16)));
-	unsigned long long rx_dropped		__attribute__ ((aligned (16)));
-	unsigned long long tx_dropped		__attribute__ ((aligned (16)));
-	unsigned long long rx_fifo_errors	__attribute__ ((aligned (16)));
-	unsigned long long tx_fifo_errors	__attribute__ ((aligned (16)));
-	unsigned long long rx_frame_errors	__attribute__ ((aligned (16)));
-	unsigned long long tx_carrier_errors	__attribute__ ((aligned (16)));
-	char	      interface[MAX_IFACE_LEN]	__attribute__ ((aligned (16)));
+	unsigned long long collisions;
+	unsigned long long rx_errors;
+	unsigned long long tx_errors;
+	unsigned long long rx_dropped;
+	unsigned long long tx_dropped;
+	unsigned long long rx_fifo_errors;
+	unsigned long long tx_fifo_errors;
+	unsigned long long rx_frame_errors;
+	unsigned long long tx_carrier_errors;
+	char		   interface[MAX_IFACE_LEN];
 };
 
 #define STATS_NET_EDEV_SIZE	(sizeof(struct stats_net_edev))
+#define STATS_NET_EDEV_SIZE2CMP	(STATS_NET_EDEV_SIZE - MAX_IFACE_LEN)
+#define STATS_NET_EDEV_ULL	9
+#define STATS_NET_EDEV_UL	0
+#define STATS_NET_EDEV_U	0
 
 /* Structure for NFS client statistics */
 struct stats_net_nfs {
-	unsigned int  nfs_rpccnt	__attribute__ ((aligned (4)));
-	unsigned int  nfs_rpcretrans	__attribute__ ((packed));
-	unsigned int  nfs_readcnt	__attribute__ ((packed));
-	unsigned int  nfs_writecnt	__attribute__ ((packed));
-	unsigned int  nfs_accesscnt	__attribute__ ((packed));
-	unsigned int  nfs_getattcnt	__attribute__ ((packed));
+	unsigned int nfs_rpccnt;
+	unsigned int nfs_rpcretrans;
+	unsigned int nfs_readcnt;
+	unsigned int nfs_writecnt;
+	unsigned int nfs_accesscnt;
+	unsigned int nfs_getattcnt;
 };
 
 #define STATS_NET_NFS_SIZE	(sizeof(struct stats_net_nfs))
+#define STATS_NET_NFS_ULL	0
+#define STATS_NET_NFS_UL	0
+#define STATS_NET_NFS_U		6
 
 /* Structure for NFS server statistics */
 struct stats_net_nfsd {
-	unsigned int  nfsd_rpccnt	__attribute__ ((aligned (4)));
-	unsigned int  nfsd_rpcbad	__attribute__ ((packed));
-	unsigned int  nfsd_netcnt	__attribute__ ((packed));
-	unsigned int  nfsd_netudpcnt	__attribute__ ((packed));
-	unsigned int  nfsd_nettcpcnt	__attribute__ ((packed));
-	unsigned int  nfsd_rchits	__attribute__ ((packed));
-	unsigned int  nfsd_rcmisses	__attribute__ ((packed));
-	unsigned int  nfsd_readcnt	__attribute__ ((packed));
-	unsigned int  nfsd_writecnt	__attribute__ ((packed));
-	unsigned int  nfsd_accesscnt	__attribute__ ((packed));
-	unsigned int  nfsd_getattcnt	__attribute__ ((packed));
+	unsigned int nfsd_rpccnt;
+	unsigned int nfsd_rpcbad;
+	unsigned int nfsd_netcnt;
+	unsigned int nfsd_netudpcnt;
+	unsigned int nfsd_nettcpcnt;
+	unsigned int nfsd_rchits;
+	unsigned int nfsd_rcmisses;
+	unsigned int nfsd_readcnt;
+	unsigned int nfsd_writecnt;
+	unsigned int nfsd_accesscnt;
+	unsigned int nfsd_getattcnt;
 };
 
 #define STATS_NET_NFSD_SIZE	(sizeof(struct stats_net_nfsd))
+#define STATS_NET_NFSD_ULL	0
+#define STATS_NET_NFSD_UL	0
+#define STATS_NET_NFSD_U	11
 
 /* Structure for IPv4 sockets statistics */
 struct stats_net_sock {
-	unsigned int  sock_inuse	__attribute__ ((aligned (4)));
-	unsigned int  tcp_inuse		__attribute__ ((packed));
-	unsigned int  tcp_tw		__attribute__ ((packed));
-	unsigned int  udp_inuse		__attribute__ ((packed));
-	unsigned int  raw_inuse		__attribute__ ((packed));
-	unsigned int  frag_inuse	__attribute__ ((packed));
+	unsigned int sock_inuse;
+	unsigned int tcp_inuse;
+	unsigned int tcp_tw;
+	unsigned int udp_inuse;
+	unsigned int raw_inuse;
+	unsigned int frag_inuse;
 };
 
 #define STATS_NET_SOCK_SIZE	(sizeof(struct stats_net_sock))
+#define STATS_NET_SOCK_ULL	0
+#define STATS_NET_SOCK_UL	0
+#define STATS_NET_SOCK_U	6
 
 /* Structure for IP statistics */
 struct stats_net_ip {
-	unsigned long long InReceives		__attribute__ ((aligned (16)));
-	unsigned long long ForwDatagrams	__attribute__ ((aligned (16)));
-	unsigned long long InDelivers		__attribute__ ((aligned (16)));
-	unsigned long long OutRequests		__attribute__ ((aligned (16)));
-	unsigned long long ReasmReqds		__attribute__ ((aligned (16)));
-	unsigned long long ReasmOKs		__attribute__ ((aligned (16)));
-	unsigned long long FragOKs		__attribute__ ((aligned (16)));
-	unsigned long long FragCreates		__attribute__ ((aligned (16)));
+	unsigned long long InReceives;
+	unsigned long long ForwDatagrams;
+	unsigned long long InDelivers;
+	unsigned long long OutRequests;
+	unsigned long long ReasmReqds;
+	unsigned long long ReasmOKs;
+	unsigned long long FragOKs;
+	unsigned long long FragCreates;
 };
 
 #define STATS_NET_IP_SIZE	(sizeof(struct stats_net_ip))
+#define STATS_NET_IP_ULL	8
+#define STATS_NET_IP_UL		0
+#define STATS_NET_IP_U		0
 
 /* Structure for IP errors statistics */
 struct stats_net_eip {
-	unsigned long long InHdrErrors		__attribute__ ((aligned (16)));
-	unsigned long long InAddrErrors		__attribute__ ((aligned (16)));
-	unsigned long long InUnknownProtos	__attribute__ ((aligned (16)));
-	unsigned long long InDiscards		__attribute__ ((aligned (16)));
-	unsigned long long OutDiscards		__attribute__ ((aligned (16)));
-	unsigned long long OutNoRoutes		__attribute__ ((aligned (16)));
-	unsigned long long ReasmFails		__attribute__ ((aligned (16)));
-	unsigned long long FragFails		__attribute__ ((aligned (16)));
+	unsigned long long InHdrErrors;
+	unsigned long long InAddrErrors;
+	unsigned long long InUnknownProtos;
+	unsigned long long InDiscards;
+	unsigned long long OutDiscards;
+	unsigned long long OutNoRoutes;
+	unsigned long long ReasmFails;
+	unsigned long long FragFails;
 };
 
 #define STATS_NET_EIP_SIZE	(sizeof(struct stats_net_eip))
+#define STATS_NET_EIP_ULL	8
+#define STATS_NET_EIP_UL	0
+#define STATS_NET_EIP_U		0
 
 /* Structure for ICMP statistics */
 struct stats_net_icmp {
@@ -358,6 +421,9 @@ struct stats_net_icmp {
 };
 
 #define STATS_NET_ICMP_SIZE	(sizeof(struct stats_net_icmp))
+#define STATS_NET_ICMP_ULL	0
+#define STATS_NET_ICMP_UL	14
+#define STATS_NET_ICMP_U	0
 
 /* Structure for ICMP error message statistics */
 struct stats_net_eicmp {
@@ -376,6 +442,9 @@ struct stats_net_eicmp {
 };
 
 #define STATS_NET_EICMP_SIZE	(sizeof(struct stats_net_eicmp))
+#define STATS_NET_EICMP_ULL	0
+#define STATS_NET_EICMP_UL	12
+#define STATS_NET_EICMP_U	0
 
 /* Structure for TCP statistics */
 struct stats_net_tcp {
@@ -386,6 +455,9 @@ struct stats_net_tcp {
 };
 
 #define STATS_NET_TCP_SIZE	(sizeof(struct stats_net_tcp))
+#define STATS_NET_TCP_ULL	0
+#define STATS_NET_TCP_UL	4
+#define STATS_NET_TCP_U		0
 
 /* Structure for TCP errors statistics */
 struct stats_net_etcp {
@@ -397,6 +469,9 @@ struct stats_net_etcp {
 };
 
 #define STATS_NET_ETCP_SIZE	(sizeof(struct stats_net_etcp))
+#define STATS_NET_ETCP_ULL	0
+#define STATS_NET_ETCP_UL	5
+#define STATS_NET_ETCP_U	0
 
 /* Structure for UDP statistics */
 struct stats_net_udp {
@@ -407,39 +482,61 @@ struct stats_net_udp {
 };
 
 #define STATS_NET_UDP_SIZE	(sizeof(struct stats_net_udp))
+#define STATS_NET_UDP_ULL	0
+#define STATS_NET_UDP_UL	4
+#define STATS_NET_UDP_U		0
+
+/* Structure for IPv6 sockets statistics */
+struct stats_net_sock6 {
+	unsigned int tcp6_inuse;
+	unsigned int udp6_inuse;
+	unsigned int raw6_inuse;
+	unsigned int frag6_inuse;
+};
+
+#define STATS_NET_SOCK6_SIZE	(sizeof(struct stats_net_sock6))
+#define STATS_NET_SOCK6_ULL	0
+#define STATS_NET_SOCK6_UL	0
+#define STATS_NET_SOCK6_U	4
 
 /* Structure for IPv6 statistics */
 struct stats_net_ip6 {
-	unsigned long long InReceives6		__attribute__ ((aligned (16)));
-	unsigned long long OutForwDatagrams6	__attribute__ ((aligned (16)));
-	unsigned long long InDelivers6		__attribute__ ((aligned (16)));
-	unsigned long long OutRequests6		__attribute__ ((aligned (16)));
-	unsigned long long ReasmReqds6		__attribute__ ((aligned (16)));
-	unsigned long long ReasmOKs6		__attribute__ ((aligned (16)));
-	unsigned long long InMcastPkts6		__attribute__ ((aligned (16)));
-	unsigned long long OutMcastPkts6	__attribute__ ((aligned (16)));
-	unsigned long long FragOKs6		__attribute__ ((aligned (16)));
-	unsigned long long FragCreates6		__attribute__ ((aligned (16)));
+	unsigned long long InReceives6;
+	unsigned long long OutForwDatagrams6;
+	unsigned long long InDelivers6;
+	unsigned long long OutRequests6;
+	unsigned long long ReasmReqds6;
+	unsigned long long ReasmOKs6;
+	unsigned long long InMcastPkts6;
+	unsigned long long OutMcastPkts6;
+	unsigned long long FragOKs6;
+	unsigned long long FragCreates6;
 };
 
 #define STATS_NET_IP6_SIZE	(sizeof(struct stats_net_ip6))
+#define STATS_NET_IP6_ULL	10
+#define STATS_NET_IP6_UL	0
+#define STATS_NET_IP6_U		0
 
 /* Structure for IPv6 errors statistics */
 struct stats_net_eip6 {
-	unsigned long long InHdrErrors6		__attribute__ ((aligned (16)));
-	unsigned long long InAddrErrors6	__attribute__ ((aligned (16)));
-	unsigned long long InUnknownProtos6	__attribute__ ((aligned (16)));
-	unsigned long long InTooBigErrors6	__attribute__ ((aligned (16)));
-	unsigned long long InDiscards6		__attribute__ ((aligned (16)));
-	unsigned long long OutDiscards6		__attribute__ ((aligned (16)));
-	unsigned long long InNoRoutes6		__attribute__ ((aligned (16)));
-	unsigned long long OutNoRoutes6		__attribute__ ((aligned (16)));
-	unsigned long long ReasmFails6		__attribute__ ((aligned (16)));
-	unsigned long long FragFails6		__attribute__ ((aligned (16)));
-	unsigned long long InTruncatedPkts6	__attribute__ ((aligned (16)));
+	unsigned long long InHdrErrors6;
+	unsigned long long InAddrErrors6;
+	unsigned long long InUnknownProtos6;
+	unsigned long long InTooBigErrors6;
+	unsigned long long InDiscards6;
+	unsigned long long OutDiscards6;
+	unsigned long long InNoRoutes6;
+	unsigned long long OutNoRoutes6;
+	unsigned long long ReasmFails6;
+	unsigned long long FragFails6;
+	unsigned long long InTruncatedPkts6;
 };
 
 #define STATS_NET_EIP6_SIZE	(sizeof(struct stats_net_eip6))
+#define STATS_NET_EIP6_ULL	11
+#define STATS_NET_EIP6_UL	0
+#define STATS_NET_EIP6_U	0
 
 /* Structure for ICMPv6 statistics */
 struct stats_net_icmp6 {
@@ -463,6 +560,9 @@ struct stats_net_icmp6 {
 };
 
 #define STATS_NET_ICMP6_SIZE	(sizeof(struct stats_net_icmp6))
+#define STATS_NET_ICMP6_ULL	0
+#define STATS_NET_ICMP6_UL	17
+#define STATS_NET_ICMP6_U	0
 
 /* Structure for ICMPv6 error message statistics */
 struct stats_net_eicmp6 {
@@ -480,6 +580,9 @@ struct stats_net_eicmp6 {
 };
 
 #define STATS_NET_EICMP6_SIZE	(sizeof(struct stats_net_eicmp6))
+#define STATS_NET_EICMP6_ULL	0
+#define STATS_NET_EICMP6_UL	11
+#define STATS_NET_EICMP6_U	0
 
 /* Structure for UDPv6 statistics */
 struct stats_net_udp6 {
@@ -490,16 +593,9 @@ struct stats_net_udp6 {
 };
 
 #define STATS_NET_UDP6_SIZE	(sizeof(struct stats_net_udp6))
-
-/* Structure for IPv6 sockets statistics */
-struct stats_net_sock6 {
-	unsigned int  tcp6_inuse	__attribute__ ((aligned (4)));
-	unsigned int  udp6_inuse	__attribute__ ((packed));
-	unsigned int  raw6_inuse	__attribute__ ((packed));
-	unsigned int  frag6_inuse	__attribute__ ((packed));
-};
-
-#define STATS_NET_SOCK6_SIZE	(sizeof(struct stats_net_sock6))
+#define STATS_NET_UDP6_ULL	0
+#define STATS_NET_UDP6_UL	4
+#define STATS_NET_UDP6_U	0
 
 /*
  * Structure for CPU frequency statistics.
@@ -511,14 +607,22 @@ struct stats_pwr_cpufreq {
 };
 
 #define STATS_PWR_CPUFREQ_SIZE	(sizeof(struct stats_pwr_cpufreq))
+#define STATS_PWR_CPUFREQ_ULL	0
+#define STATS_PWR_CPUFREQ_UL	1
+#define STATS_PWR_CPUFREQ_U	0
 
 /* Structure for hugepages statistics */
 struct stats_huge {
-	unsigned long frhkb			__attribute__ ((aligned (8)));
-	unsigned long tlhkb			__attribute__ ((aligned (8)));
+	unsigned long long frhkb;
+	unsigned long long tlhkb;
+	unsigned long long rsvdhkb;
+	unsigned long long surphkb;
 };
 
-#define STATS_HUGE_SIZE	(sizeof(struct stats_memory))
+#define STATS_HUGE_SIZE	(sizeof(struct stats_huge))
+#define STATS_HUGE_ULL	4
+#define STATS_HUGE_UL	0
+#define STATS_HUGE_U	0
 
 /*
  * Structure for weighted CPU frequency statistics.
@@ -526,38 +630,48 @@ struct stats_huge {
  * Following structures are for each individual CPU (0, 1, etc.)
  */
 struct stats_pwr_wghfreq {
-	unsigned long long 	time_in_state	__attribute__ ((aligned (16)));
-	unsigned long 		freq		__attribute__ ((aligned (16)));
+	unsigned long long time_in_state;
+	unsigned long 	   freq		__attribute__ ((aligned (8)));
 };
 
 #define STATS_PWR_WGHFREQ_SIZE	(sizeof(struct stats_pwr_wghfreq))
+#define STATS_PWR_WGHFREQ_ULL	1
+#define STATS_PWR_WGHFREQ_UL	1
+#define STATS_PWR_WGHFREQ_U	0
 
 /*
  * Structure for USB devices plugged into the system.
  */
 struct stats_pwr_usb {
-	unsigned int  bus_nr				__attribute__ ((aligned (4)));
-	unsigned int  vendor_id				__attribute__ ((packed));
-	unsigned int  product_id			__attribute__ ((packed));
-	unsigned int  bmaxpower				__attribute__ ((packed));
-	char	      manufacturer[MAX_MANUF_LEN];
-	char	      product[MAX_PROD_LEN];
+	unsigned int bus_nr;
+	unsigned int vendor_id;
+	unsigned int product_id;
+	unsigned int bmaxpower;
+	char	     manufacturer[MAX_MANUF_LEN];
+	char	     product[MAX_PROD_LEN];
 };
 
 #define STATS_PWR_USB_SIZE	(sizeof(struct stats_pwr_usb))
+#define STATS_PWR_USB_ULL	0
+#define STATS_PWR_USB_UL	0
+#define STATS_PWR_USB_U		4
 
 /* Structure for filesystems statistics */
 struct stats_filesystem {
-	unsigned long long f_blocks		__attribute__ ((aligned (16)));
-	unsigned long long f_bfree		__attribute__ ((aligned (16)));
-	unsigned long long f_bavail		__attribute__ ((aligned (16)));
-	unsigned long long f_files		__attribute__ ((aligned (16)));
-	unsigned long long f_ffree		__attribute__ ((aligned (16)));
-	char 		   fs_name[MAX_FS_LEN]	__attribute__ ((aligned (16)));
+	unsigned long long f_blocks;
+	unsigned long long f_bfree;
+	unsigned long long f_bavail;
+	unsigned long long f_files;
+	unsigned long long f_ffree;
+	char 		   fs_name[MAX_FS_LEN];
 	char 		   mountp[MAX_FS_LEN];
 };
 
-#define STATS_FILESYSTEM_SIZE	(sizeof(struct stats_filesystem))
+#define STATS_FILESYSTEM_SIZE		(sizeof(struct stats_filesystem))
+#define STATS_FILESYSTEM_SIZE2CMP	(STATS_FILESYSTEM_SIZE - 2 * MAX_FS_LEN)
+#define STATS_FILESYSTEM_ULL		5
+#define STATS_FILESYSTEM_UL		0
+#define STATS_FILESYSTEM_U		0
 
 /* Structure for Fibre Channel HBA statistics */
 struct stats_fchost {
@@ -569,17 +683,23 @@ struct stats_fchost {
 };
 
 #define STATS_FCHOST_SIZE	(sizeof(struct stats_fchost))
+#define STATS_FCHOST_ULL	0
+#define STATS_FCHOST_UL		4
+#define STATS_FCHOST_U		0
 
 /* Structure for softnet statistics */
 struct stats_softnet {
-	unsigned int processed		__attribute__ ((aligned (4)));
-	unsigned int dropped		__attribute__ ((packed));
-	unsigned int time_squeeze	__attribute__ ((packed));
-	unsigned int received_rps	__attribute__ ((packed));
-	unsigned int flow_limit		__attribute__ ((packed));
+	unsigned int processed;
+	unsigned int dropped;
+	unsigned int time_squeeze;
+	unsigned int received_rps;
+	unsigned int flow_limit;
 };
 
 #define STATS_SOFTNET_SIZE	(sizeof(struct stats_softnet))
+#define STATS_SOFTNET_ULL	0
+#define STATS_SOFTNET_UL	0
+#define STATS_SOFTNET_U		5
 
 /*
  ***************************************************************************
@@ -587,83 +707,90 @@ struct stats_softnet {
  ***************************************************************************
  */
 
-void oct2chr
-	(char *);
-void read_stat_cpu
-	(struct stats_cpu *, int, unsigned long long *, unsigned long long *);
-void read_stat_irq
-	(struct stats_irq *, int);
-void read_meminfo
+void compute_ext_disk_stats
+	(struct stats_disk *, struct stats_disk *, unsigned long long,
+	 struct ext_disk_stats *);
+unsigned long long get_per_cpu_interval
+	(struct stats_cpu *, struct stats_cpu *);
+__nr_t read_stat_cpu
+	(struct stats_cpu *, __nr_t);
+__nr_t read_stat_irq
+	(struct stats_irq *, __nr_t);
+__nr_t read_meminfo
 	(struct stats_memory *);
 void read_uptime
 	(unsigned long long *);
-void read_stat_pcsw
+#ifdef SOURCE_SADC
+void oct2chr
+	(char *);
+__nr_t read_stat_pcsw
 	(struct stats_pcsw *);
-void read_loadavg
+__nr_t read_loadavg
 	(struct stats_queue *);
-void read_vmstat_swap
+__nr_t read_vmstat_swap
 	(struct stats_swap *);
-void read_vmstat_paging
+__nr_t read_vmstat_paging
 	(struct stats_paging *);
-void read_diskstats_io
+__nr_t read_diskstats_io
 	(struct stats_io *);
-void read_diskstats_disk
-	(struct stats_disk *, int, int);
-void read_tty_driver_serial
-	(struct stats_serial *, int);
-void read_kernel_tables
+__nr_t read_diskstats_disk
+	(struct stats_disk *, __nr_t, int);
+__nr_t read_tty_driver_serial
+	(struct stats_serial *, __nr_t);
+__nr_t read_kernel_tables
 	(struct stats_ktables *);
-int read_net_dev
-	(struct stats_net_dev *, int);
+__nr_t read_net_dev
+	(struct stats_net_dev *, __nr_t);
 void read_if_info
 	(struct stats_net_dev *, int);
-void read_net_edev
-	(struct stats_net_edev *, int);
-void read_net_nfs
+__nr_t read_net_edev
+	(struct stats_net_edev *, __nr_t);
+__nr_t read_net_nfs
 	(struct stats_net_nfs *);
-void read_net_nfsd
+__nr_t read_net_nfsd
 	(struct stats_net_nfsd *);
-void read_net_sock
+__nr_t read_net_sock
 	(struct stats_net_sock *);
-void read_net_ip
+__nr_t read_net_ip
 	(struct stats_net_ip *);
-void read_net_eip
+__nr_t read_net_eip
 	(struct stats_net_eip *);
-void read_net_icmp
+__nr_t read_net_icmp
 	(struct stats_net_icmp *);
-void read_net_eicmp
+__nr_t read_net_eicmp
 	(struct stats_net_eicmp *);
-void read_net_tcp
+__nr_t read_net_tcp
 	(struct stats_net_tcp *);
-void read_net_etcp
+__nr_t read_net_etcp
 	(struct stats_net_etcp *);
-void read_net_udp
+__nr_t read_net_udp
 	(struct stats_net_udp *);
-void read_net_sock6
+__nr_t read_net_sock6
 	(struct stats_net_sock6 *);
-void read_net_ip6
+__nr_t read_net_ip6
 	(struct stats_net_ip6 *);
-void read_net_eip6
+__nr_t read_net_eip6
 	(struct stats_net_eip6 *);
-void read_net_icmp6
+__nr_t read_net_icmp6
 	(struct stats_net_icmp6 *);
-void read_net_eicmp6
+__nr_t read_net_eicmp6
 	(struct stats_net_eicmp6 *);
-void read_net_udp6
+__nr_t read_net_udp6
 	(struct stats_net_udp6 *);
-void read_cpuinfo
-	(struct stats_pwr_cpufreq *, int);
-void read_meminfo_huge
+__nr_t read_cpuinfo
+	(struct stats_pwr_cpufreq *, __nr_t);
+__nr_t read_meminfo_huge
 	(struct stats_huge *);
-void read_time_in_state
-	(struct stats_pwr_wghfreq *, int, int);
-void read_bus_usb_dev
-	(struct stats_pwr_usb *, int);
-void read_filesystem
-	(struct stats_filesystem *, int);
-void read_fchost
-	(struct stats_fchost *, int);
-void read_softnet
-	(struct stats_softnet *, int);
+__nr_t read_cpu_wghfreq
+	(struct stats_pwr_wghfreq *, __nr_t, __nr_t);
+__nr_t read_bus_usb_dev
+	(struct stats_pwr_usb *, __nr_t);
+__nr_t read_filesystem
+	(struct stats_filesystem *, __nr_t);
+__nr_t read_fchost
+	(struct stats_fchost *, __nr_t);
+int read_softnet
+	(struct stats_softnet *, __nr_t, unsigned char []);
+#endif /* SOURCE_SADC */
 
 #endif /* _RD_STATS_H */
