@@ -711,6 +711,13 @@ __print_funct_t render_io_stats(struct activity *a, int isdb, char *pre,
 	       S_VALUE(sip->dk_drive_wio, sic->dk_drive_wio, itv),
 	       NULL);
 
+	render(isdb, pre, PT_NOFLAG,
+	       "-\tdtps", NULL, NULL,
+	       NOVAL,
+	       sic->dk_drive_dio < sip->dk_drive_dio ? 0.0 :
+	       S_VALUE(sip->dk_drive_dio, sic->dk_drive_dio, itv),
+	       NULL);
+
 	render(isdb, pre, PT_NOFLAG,
 	       "-\tbread/s", NULL, NULL,
 	       NOVAL,
@@ -718,12 +725,19 @@ __print_funct_t render_io_stats(struct activity *a, int isdb, char *pre,
 	       S_VALUE(sip->dk_drive_rblk, sic->dk_drive_rblk, itv),
 	       NULL);
 
-	render(isdb, pre, pt_newlin,
+	render(isdb, pre, PT_NOFLAG,
 	       "-\tbwrtn/s", NULL, NULL,
 	       NOVAL,
 	       sic->dk_drive_wblk < sip->dk_drive_wblk ? 0.0 :
 	       S_VALUE(sip->dk_drive_wblk, sic->dk_drive_wblk, itv),
 	       NULL);
+
+	render(isdb, pre, pt_newlin,
+	       "-\tbdscd/s", NULL, NULL,
+	       NOVAL,
+	       sic->dk_drive_dblk < sip->dk_drive_dblk ? 0.0 :
+	       S_VALUE(sip->dk_drive_dblk, sic->dk_drive_dblk, itv),
+	       NULL);
 }
 
 /*
@@ -1110,6 +1124,13 @@ __print_funct_t render_disk_stats(struct activity *a, int isdb, char *pre,
 		       S_VALUE(sdp->wr_sect, sdc->wr_sect, itv) / 2,
 		       NULL);
 
+		render(isdb, pre, PT_NOFLAG,
+		       "%s\tdkB/s", NULL,
+		       cons(sv, dev_name, NULL),
+		       NOVAL,
+		       S_VALUE(sdp->dc_sect, sdc->dc_sect, itv) / 2,
+		       NULL);
+
 		render(isdb, pre, PT_NOFLAG,
 		       "%s\tareq-sz", NULL,
 		       cons(sv, dev_name, NULL),
@@ -1131,13 +1152,6 @@ __print_funct_t render_disk_stats(struct activity *a, int isdb, char *pre,
 		       xds.await,
 		       NULL);
 
-		render(isdb, pre, PT_NOFLAG,
-		       "%s\tsvctm", NULL,
-		       cons(sv, dev_name, NULL),
-		       NOVAL,
-		       xds.svctm,
-		       NULL);
-
 		render(isdb, pre, pt_newlin,
 		       "%s\t%%util", NULL,
 		       cons(sv, dev_name, NULL),
@@ -2730,11 +2744,19 @@ __print_funct_t render_huge_stats(struct activity *a, int isdb, char *pre,
 	       "-\tkbhugused", NULL, NULL,
 	       smc->tlhkb - smc->frhkb, DNOVAL, NULL);
 
-	render(isdb, pre, pt_newlin,
+	render(isdb, pre, PT_NOFLAG,
 	       "-\t%hugused", NULL, NULL, NOVAL,
 	       smc->tlhkb ?
 	       SP_VALUE(smc->frhkb, smc->tlhkb, smc->tlhkb) :
 	       0.0, NULL);
+
+	render(isdb, pre, PT_USEINT,
+	       "-\tkbhugrsvd", NULL, NULL,
+	       smc->rsvdhkb, DNOVAL, NULL);
+
+	render(isdb, pre, PT_USEINT | pt_newlin,
+	       "-\tkbhugsurp", NULL, NULL,
+	       smc->surphkb, DNOVAL, NULL);
 }
 
 /*
