@@ -804,8 +804,6 @@ void get_itv_value(struct record_header *record_hdr_curr,
 void get_file_timestamp_struct(unsigned int flags, struct tm *rectime,
 			       struct file_header *file_hdr)
 {
-	struct tm *loc_t;
-
 	if (PRINT_TRUE_TIME(flags)) {
 		/* Get local time. This is just to fill fields with a default value. */
 		get_time(rectime, 0);
@@ -821,9 +819,7 @@ void get_file_timestamp_struct(unsigned int flags, struct tm *rectime,
 		mktime(rectime);
 	}
 	else {
-		if ((loc_t = localtime((const time_t *) &file_hdr->sa_ust_time)) != NULL) {
-			*rectime = *loc_t;
-		}
+		localtime_r((const time_t *) &file_hdr->sa_ust_time, rectime);
 	}
 }
 
@@ -1096,7 +1092,8 @@ int check_disk_reg(struct activity *a, int curr, int ref, int pos)
 			 */
 			if ((sdc->nr_ios < sdp->nr_ios) &&
 			    (sdc->rd_sect < sdp->rd_sect) &&
-			    (sdc->wr_sect < sdp->wr_sect))
+			    (sdc->wr_sect < sdp->wr_sect) &&
+			    (sdc->dc_sect < sdp->dc_sect))
 				/* Same device registered again */
 				return -2;
 
@@ -1281,17 +1278,20 @@ void swap_struct(unsigned int types_nr[], void *ps, int is64bit)
  *		size of the structure *read from file*.
  * @g_size	Size of the structure expected by current sysstat version.
  * @b_size	Size of the buffer pointed by @ps.
+ *
+ * RETURNS:
+ * -1 if an error has been encountered, or 0 otherwise.
  ***************************************************************************
  */
-void remap_struct(unsigned int gtypes_nr[], unsigned int ftypes_nr[],
-		  void *ps, unsigned int f_size, unsigned int g_size, size_t b_size)
+int remap_struct(unsigned int gtypes_nr[], unsigned int ftypes_nr[],
+		 void *ps, unsigned int f_size, unsigned int g_size, size_t b_size)
 {
 	int d;
 	size_t n;
 
 	/* Sanity check */
 	if (MAP_SIZE(ftypes_nr) > f_size)
-		return;
+		return -1;
 
 	/* Remap [unsigned] long fields */
 	d = gtypes_nr[0] - ftypes_nr[0];
@@ -1301,7 +1301,7 @@ void remap_struct(unsigned int gtypes_nr[], unsigned int ftypes_nr[],
 		if ((ftypes_nr[0] * ULL_ALIGNMENT_WIDTH >= b_size) ||
 		    (gtypes_nr[0] * ULL_ALIGNMENT_WIDTH + n > b_size) ||
 		    (ftypes_nr[0] * ULL_ALIGNMENT_WIDTH + n > b_size))
-			return;
+			return -1;
 		memmove(((char *) ps) + gtypes_nr[0] * ULL_ALIGNMENT_WIDTH,
 			((char *) ps) + ftypes_nr[0] * ULL_ALIGNMENT_WIDTH, n);
 		if (d > 0) {
@@ -1322,7 +1322,7 @@ void remap_struct(unsigned int gtypes_nr[], unsigned int ftypes_nr[],
 		     gtypes_nr[1] * UL_ALIGNMENT_WIDTH + n > b_size) ||
 		    (gtypes_nr[0] * ULL_ALIGNMENT_WIDTH +
 		     ftypes_nr[1] * UL_ALIGNMENT_WIDTH + n > b_size))
-			return;
+			return -1;
 		memmove(((char *) ps) + gtypes_nr[0] * ULL_ALIGNMENT_WIDTH
 				      + gtypes_nr[1] * UL_ALIGNMENT_WIDTH,
 			((char *) ps) + gtypes_nr[0] * ULL_ALIGNMENT_WIDTH
@@ -1351,7 +1351,7 @@ void remap_struct(unsigned int gtypes_nr[], unsigned int ftypes_nr[],
 		    (gtypes_nr[0] * ULL_ALIGNMENT_WIDTH +
 		     gtypes_nr[1] * UL_ALIGNMENT_WIDTH +
 		     ftypes_nr[2] * U_ALIGNMENT_WIDTH + n > b_size))
-			return;
+			return -1;
 		memmove(((char *) ps) + gtypes_nr[0] * ULL_ALIGNMENT_WIDTH
 				      + gtypes_nr[1] * UL_ALIGNMENT_WIDTH
 				      + gtypes_nr[2] * U_ALIGNMENT_WIDTH,
@@ -1365,6 +1365,7 @@ void remap_struct(unsigned int gtypes_nr[], unsigned int ftypes_nr[],
 			       0, d * U_ALIGNMENT_WIDTH);
 		}
 	}
+	return 0;
 }
 
 /*
@@ -1377,12 +1378,16 @@ void remap_struct(unsigned int gtypes_nr[], unsigned int ftypes_nr[],
  * @size	Number of bytes to read.
  * @mode	If set to HARD_SIZE, indicate that an EOF should be considered
  * 		as an error.
+ * @oneof	Set to UEOF_CONT if an unexpected end of file should not make
+ *		sadf stop. Default behavior is to stop on unexpected EOF.
  *
  * RETURNS:
- * 1 if EOF has been reached, 0 otherwise.
+ * 1 if EOF has been reached,
+ * 2 if an unexpected EOF has been reached (and sadf was told to continue),
+ * 0 otherwise.
  ***************************************************************************
  */
-int sa_fread(int ifd, void *buffer, size_t size, int mode)
+int sa_fread(int ifd, void *buffer, size_t size, int mode, int oneof)
 {
 	ssize_t n;
 
@@ -1398,6 +1403,8 @@ int sa_fread(int ifd, void *buffer, size_t size, int mode)
 
 	if (n < size) {
 		fprintf(stderr, _("End of system activity file unexpected\n"));
+		if (oneof == UEOF_CONT)
+			return 2;
 		close(ifd);
 		exit(2);
 	}
@@ -1419,26 +1426,33 @@ int sa_fread(int ifd, void *buffer, size_t size, int mode)
  * @endian_mismatch
  *		TRUE if data read from file don't match current machine's
  *		endianness.
+ * @oneof	Set to EOF_CONT if an unexpected end of file should not make
+ *		sadf stop. Default behavior is to stop on unexpected EOF.
  * @b_size	@buffer size.
  *
  * OUT:
  * @record_hdr	Record header for current sample.
  *
  * RETURNS:
- * 1 if EOF has been reached, 0 otherwise.
+ * 1 if EOF has been reached,
+ * 2 if an error has been encountered (e.g. unexpected EOF),
+ * 0 otherwise.
  ***************************************************************************
  */
 int read_record_hdr(int ifd, void *buffer, struct record_header *record_hdr,
 		    struct file_header *file_hdr, int arch_64, int endian_mismatch,
-		    size_t b_size)
+		    int oneof, size_t b_size)
 {
-	if (sa_fread(ifd, buffer, (size_t) file_hdr->rec_size, SOFT_SIZE))
+	int rc;
+
+	if ((rc = sa_fread(ifd, buffer, (size_t) file_hdr->rec_size, SOFT_SIZE, oneof)) != 0)
 		/* End of sa data file */
-		return 1;
+		return rc;
 
 	/* Remap record header structure to that expected by current version */
-	remap_struct(rec_types_nr, file_hdr->rec_types_nr, buffer,
-		     file_hdr->rec_size, RECORD_HEADER_SIZE, b_size);
+	if (remap_struct(rec_types_nr, file_hdr->rec_types_nr, buffer,
+			 file_hdr->rec_size, RECORD_HEADER_SIZE, b_size) < 0)
+		return 2;
 	memcpy(record_hdr, buffer, RECORD_HEADER_SIZE);
 
 	/* Normalize endianness */
@@ -1506,7 +1520,7 @@ __nr_t read_nr_value(int ifd, char *file, struct file_magic *file_magic,
 {
 	__nr_t value;
 
-	sa_fread(ifd, &value, sizeof(__nr_t), HARD_SIZE);
+	sa_fread(ifd, &value, sizeof(__nr_t), HARD_SIZE, UEOF_STOP);
 
 	/* Normalize endianness for file_activity structures */
 	if (endian_mismatch) {
@@ -1542,11 +1556,18 @@ __nr_t read_nr_value(int ifd, char *file, struct file_magic *file_magic,
  * @dfile	Name of system activity data file.
  * @file_magic	file_magic structure containing data read from file magic
  *		header.
+ * @oneof	Set to UEOF_CONT if an unexpected end of file should not make
+ *		sadf stop. Default behavior is to stop on unexpected EOF.
+ *
+ * RETURNS:
+ * 2 if an error has been encountered (e.g. unexpected EOF),
+ * 0 otherwise.
  ***************************************************************************
  */
-void read_file_stat_bunch(struct activity *act[], int curr, int ifd, int act_nr,
-			  struct file_activity *file_actlst, int endian_mismatch,
-			  int arch_64, char *dfile, struct file_magic *file_magic)
+int read_file_stat_bunch(struct activity *act[], int curr, int ifd, int act_nr,
+			 struct file_activity *file_actlst, int endian_mismatch,
+			 int arch_64, char *dfile, struct file_magic *file_magic,
+			 int oneof)
 {
 	int i, j, p;
 	struct file_activity *fal = file_actlst;
@@ -1582,6 +1603,8 @@ void read_file_stat_bunch(struct activity *act[], int curr, int ifd, int act_nr,
 				if (lseek(ifd, offset, SEEK_CUR) < offset) {
 					close(ifd);
 					perror("lseek");
+					if (oneof == UEOF_CONT)
+						return 2;
 					exit(2);
 				}
 			}
@@ -1618,8 +1641,10 @@ void read_file_stat_bunch(struct activity *act[], int curr, int ifd, int act_nr,
 		    (act[p]->msize > act[p]->fsize)) {
 
 			for (j = 0; j < (nr_value * act[p]->nr2); j++) {
-				sa_fread(ifd, (char *) act[p]->buf[curr] + j * act[p]->msize,
-					 (size_t) act[p]->fsize, HARD_SIZE);
+				if (sa_fread(ifd, (char *) act[p]->buf[curr] + j * act[p]->msize,
+					 (size_t) act[p]->fsize, HARD_SIZE, oneof) > 0)
+					/* Unexpected EOF */
+					return 2;
 			}
 		}
 		else if (nr_value > 0) {
@@ -1627,8 +1652,11 @@ void read_file_stat_bunch(struct activity *act[], int curr, int ifd, int act_nr,
 			 * Note: If msize was smaller than fsize,
 			 * then it has been set to fsize in check_file_actlst().
 			 */
-			sa_fread(ifd, act[p]->buf[curr],
-				 (size_t) act[p]->fsize * (size_t) nr_value * (size_t) act[p]->nr2, HARD_SIZE);
+			if (sa_fread(ifd, act[p]->buf[curr],
+				 (size_t) act[p]->fsize * (size_t) nr_value * (size_t) act[p]->nr2,
+				 HARD_SIZE, oneof) > 0)
+				/* Unexpected EOF */
+				return 2;
 		}
 		else {
 			/* nr_value == 0: Nothing to read */
@@ -1645,11 +1673,14 @@ void read_file_stat_bunch(struct activity *act[], int curr, int ifd, int act_nr,
 
 		/* Remap structure's fields to those known by current sysstat version */
 		for (j = 0; j < (nr_value * act[p]->nr2); j++) {
-			remap_struct(act[p]->gtypes_nr, act[p]->ftypes_nr,
-				     (char *) act[p]->buf[curr] + j * act[p]->msize,
-				     act[p]->fsize, act[p]->msize, act[p]->msize);
+			if (remap_struct(act[p]->gtypes_nr, act[p]->ftypes_nr,
+					 (char *) act[p]->buf[curr] + j * act[p]->msize,
+					 act[p]->fsize, act[p]->msize, act[p]->msize) < 0)
+				return 2;
 		}
 	}
+
+	return 0;
 }
 
 /*
@@ -1822,7 +1853,7 @@ void check_file_actlst(int *ifd, char *dfile, struct activity *act[],
 	SREALLOC(buffer, char, file_magic->header_size);
 
 	/* Read sa data file standard header and allocate activity list */
-	sa_fread(*ifd, buffer, (size_t) file_magic->header_size, HARD_SIZE);
+	sa_fread(*ifd, buffer, (size_t) file_magic->header_size, HARD_SIZE, UEOF_STOP);
 	/*
 	 * Data file header size (file_magic->header_size) may be greater or
 	 * smaller than FILE_HEADER_SIZE. Remap the fields of the file header
@@ -1871,14 +1902,15 @@ void check_file_actlst(int *ifd, char *dfile, struct activity *act[],
 	for (i = 0; i < file_hdr->sa_act_nr; i++, fal++) {
 
 		/* Read current file_activity structure from file */
-		sa_fread(*ifd, buffer, (size_t) file_hdr->act_size, HARD_SIZE);
+		sa_fread(*ifd, buffer, (size_t) file_hdr->act_size, HARD_SIZE, UEOF_STOP);
 		/*
 		* Data file_activity size (file_hdr->act_size) may be greater or
 		* smaller than FILE_ACTIVITY_SIZE. Remap the fields of the file's structure
 		* then copy its contents to the expected structure.
 		*/
-		remap_struct(act_types_nr, file_hdr->act_types_nr, buffer,
-			     file_hdr->act_size, FILE_ACTIVITY_SIZE, file_hdr->act_size);
+		if (remap_struct(act_types_nr, file_hdr->act_types_nr, buffer,
+			     file_hdr->act_size, FILE_ACTIVITY_SIZE, file_hdr->act_size) < 0)
+			goto format_error;
 		memcpy(fal, buffer, FILE_ACTIVITY_SIZE);
 
 		/* Normalize endianness for file_activity structures */
@@ -2640,7 +2672,7 @@ void replace_nonprintable_char(int ifd, char *comment)
 	int i;
 
 	/* Read comment */
-	sa_fread(ifd, comment, MAX_COMMENT_LEN, HARD_SIZE);
+	sa_fread(ifd, comment, MAX_COMMENT_LEN, HARD_SIZE, UEOF_STOP);
 	comment[MAX_COMMENT_LEN - 1] = '\0';
 
 	/* Replace non printable chars */
@@ -2689,8 +2721,10 @@ int sa_get_record_timestamp_struct(unsigned int l_flags, struct record_header *r
 
 	/* Fill localtime structure if given */
 	if (loctime) {
-		if ((ltm = localtime((const time_t *) &(record_hdr->ust_time))) != NULL) {
-			*loctime = *ltm;
+		ltm = localtime_r((const time_t *) &(record_hdr->ust_time), loctime);
+		if (ltm) {
+			/* Done so that we have some default values */
+			*rectime = *loctime;
 		}
 		else {
 			rc = 1;
@@ -2700,7 +2734,7 @@ int sa_get_record_timestamp_struct(unsigned int l_flags, struct record_header *r
 	/* Fill generic rectime structure */
 	if (PRINT_LOCAL_TIME(l_flags) && !ltm) {
 		/* Get local time if not already done */
-		ltm = localtime((const time_t *) &(record_hdr->ust_time));
+		ltm = localtime_r((const time_t *) &(record_hdr->ust_time), rectime);
 	}
 
 	if (!PRINT_LOCAL_TIME(l_flags) && !PRINT_TRUE_TIME(l_flags)) {
@@ -2708,14 +2742,10 @@ int sa_get_record_timestamp_struct(unsigned int l_flags, struct record_header *r
 		 * Get time in UTC
 		 * (the user doesn't want local time nor time of file's creator).
 		 */
-		ltm = gmtime((const time_t *) &(record_hdr->ust_time));
+		ltm = gmtime_r((const time_t *) &(record_hdr->ust_time), rectime);
 	}
 
-	if (ltm) {
-		/* Done even in true time mode so that we have some default values */
-		*rectime = *ltm;
-	}
-	else {
+	if (!ltm) {
 		rc = 1;
 	}
 
