@@ -195,6 +195,18 @@ void parse_sadc_S_option(char *argv[], int opt)
 				usage(argv[0]);
 			}
 		}
+		else if (!strncmp(p, "-A_", 3)) {
+			/* Unselect activity by name */
+			for (i = 0; i < NR_ACT; i++) {
+				if (!strcmp(p + 1, act[i]->name)) {
+					act[i]->options &= ~AO_COLLECTED;
+					break;
+				}
+			}
+			if (i == NR_ACT) {
+				usage(argv[0]);
+			}
+		}
 		else {
 			usage(argv[0]);
 		}
@@ -909,7 +921,7 @@ void open_ofile(int *ofd, char ofile[], int restart_mark)
 		}
 
 		if ((file_act[i].has_nr && (act[p]->f_count_index < 0)) ||
-		    (!file_act[i].has_nr && (act[p]->f_count_index >= 0))) {
+		    (!file_act[i].has_nr && (act[p]->f_count_index >=0))) {
 #ifdef DEBUG
 			fprintf(stderr, "%s: %s: has_nr=%d count_index=%d\n",
 				__FUNCTION__, act[p]->name, file_act[i].has_nr, act[p]->f_count_index);
@@ -942,11 +954,8 @@ void open_ofile(int *ofd, char ofile[], int restart_mark)
 		 * Since we are appending data to a file, set @nr_ini to the value of the file.
 		 * Stats saved in file will all be 0 for that activity if no items exist on
 		 * the machine.
-		 * NB: We must preserve the value read for A_CPU when a LINUX RESTART is inserted.
 		 */
-		if ((act[p]->id != A_CPU) || !act[p]->nr_ini) {
-			act[p]->nr_ini = file_act[i].nr;
-		}
+		act[p]->nr_ini = file_act[i].nr;
 
 		/*
 		 * Force number of sub-items to that of the file, and reallocate structures.
