@@ -34,7 +34,6 @@
 
 extern unsigned int flags;
 extern char *seps[];
-extern unsigned int id_seq[];
 
 
 /*
@@ -695,7 +694,7 @@ __printf_funct_t print_xml_header(void *parm, int action, char *dfile,
 				  struct activity *act[], unsigned int id_seq[],
 				  struct file_activity *file_actlst)
 {
-	struct tm rectime, *loc_t;
+	struct tm rectime, loc_t;
 	char cur_time[TIMESTAMP_LEN];
 	int *tab = (int *) parm;
 
@@ -727,8 +726,8 @@ __printf_funct_t print_xml_header(void *parm, int action, char *dfile,
 		strftime(cur_time, sizeof(cur_time), "%Y-%m-%d", &rectime);
 		xprintf(*tab, "<file-date>%s</file-date>", cur_time);
 
-		if ((loc_t = gmtime((const time_t *) &file_hdr->sa_ust_time)) != NULL) {
-			strftime(cur_time, sizeof(cur_time), "%T", loc_t);
+		if (gmtime_r((const time_t *) &file_hdr->sa_ust_time, &loc_t) != NULL) {
+			strftime(cur_time, sizeof(cur_time), "%T", &loc_t);
 			xprintf(*tab, "<file-utc-time>%s</file-utc-time>", cur_time);
 		}
 
@@ -763,7 +762,7 @@ __printf_funct_t print_json_header(void *parm, int action, char *dfile,
 				   struct activity *act[], unsigned int id_seq[],
 				   struct file_activity *file_actlst)
 {
-	struct tm rectime, *loc_t;
+	struct tm rectime, loc_t;
 	char cur_time[TIMESTAMP_LEN];
 	int *tab = (int *) parm;
 
@@ -785,8 +784,8 @@ __printf_funct_t print_json_header(void *parm, int action, char *dfile,
 		strftime(cur_time, sizeof(cur_time), "%Y-%m-%d", &rectime);
 		xprintf0(*tab, "\"file-date\": \"%s\"", cur_time);
 
-		if ((loc_t = gmtime((const time_t *) &file_hdr->sa_ust_time)) != NULL) {
-			strftime(cur_time, sizeof(cur_time), "%T", loc_t);
+		if (gmtime_r((const time_t *) &file_hdr->sa_ust_time, &loc_t) != NULL) {
+			strftime(cur_time, sizeof(cur_time), "%T", &loc_t);
 			printf(",\n");
 			xprintf0(*tab, "\"file-utc-time\": \"%s\"", cur_time);
 		}
@@ -822,7 +821,7 @@ __printf_funct_t print_hdr_header(void *parm, int action, char *dfile,
 				  struct file_activity *file_actlst)
 {
 	int i, p;
-	struct tm rectime, *loc_t;
+	struct tm rectime, loc_t;
 	struct file_activity *fal;
 	char cur_time[TIMESTAMP_LEN];
 
@@ -842,7 +841,7 @@ __printf_funct_t print_hdr_header(void *parm, int action, char *dfile,
 		       file_magic->upgraded);
 
 		printf(_("Host: "));
-		print_gal_header(localtime((const time_t *) &(file_hdr->sa_ust_time)),
+		print_gal_header(localtime_r((const time_t *) &(file_hdr->sa_ust_time), &rectime),
 				 file_hdr->sa_sysname, file_hdr->sa_release,
 				 file_hdr->sa_nodename, file_hdr->sa_machine,
 				 file_hdr->sa_cpu_nr > 1 ? file_hdr->sa_cpu_nr - 1 : 1,
@@ -853,9 +852,9 @@ __printf_funct_t print_hdr_header(void *parm, int action, char *dfile,
 		strftime(cur_time, sizeof(cur_time), "%Y-%m-%d", &rectime);
 		printf(_("File date: %s\n"), cur_time);
 
-		if ((loc_t = gmtime((const time_t *) &file_hdr->sa_ust_time)) != NULL) {
+		if (gmtime_r((const time_t *) &file_hdr->sa_ust_time, &loc_t) != NULL) {
 			printf(_("File time: "));
-			strftime(cur_time, sizeof(cur_time), "%T", loc_t);
+			strftime(cur_time, sizeof(cur_time), "%T", &loc_t);
 			printf("%s UTC\n", cur_time);
 		}
 
@@ -920,6 +919,7 @@ __printf_funct_t print_svg_header(void *parm, int action, char *dfile,
 				  struct file_activity *file_actlst)
 {
 	struct svg_hdr_parm *hdr_parm = (struct svg_hdr_parm *) parm;
+	struct tm rectime;
 	unsigned int height = 0, ht = 0;
 	int i, p;
 
@@ -957,7 +957,7 @@ __printf_funct_t print_svg_header(void *parm, int action, char *dfile,
 		       " fill=\"black\" stroke=\"gray\" stroke-width=\"1\">\n",
 		       SVG_T_XSIZE * (hdr_parm->views_per_row), height);
 		printf("<text x=\"0\" y=\"30\" text-anchor=\"start\" stroke=\"brown\">");
-		print_gal_header(localtime((const time_t *) &(file_hdr->sa_ust_time)),
+		print_gal_header(localtime_r((const time_t *) &(file_hdr->sa_ust_time), &rectime),
 				 file_hdr->sa_sysname, file_hdr->sa_release,
 				 file_hdr->sa_nodename, file_hdr->sa_machine,
 				 file_hdr->sa_cpu_nr > 1 ? file_hdr->sa_cpu_nr - 1 : 1,
@@ -1081,9 +1081,7 @@ __nr_t count_new_filesystem(struct activity *a, int curr)
 	for (i = 0; i < a->nr[curr]; i++) {
 		sfc = (struct stats_filesystem *) ((char *) a->buf[curr] + i * a->msize);
 
-		nr += add_list_item(&(a->item_list),
-				    DISPLAY_MOUNT(a->opt_flags) ? sfc->mountp : sfc->fs_name,
-				    MAX_FS_LEN);
+		nr += add_list_item(&(a->item_list), sfc->fs_name, MAX_FS_LEN);
 	}
 
 	return nr;
