@@ -775,13 +775,13 @@ void handle_curr_act_stats(int ifd, off_t fpos, int *curr, long *cnt, int *eosaf
 		 * Start with reading current sample's record header.
 		 */
 		*eosaf = read_record_hdr(ifd, rec_hdr_tmp, &record_hdr[*curr],
-					 &file_hdr, arch_64, endian_mismatch, b_size);
+					 &file_hdr, arch_64, endian_mismatch, UEOF_STOP, b_size);
 		rtype = record_hdr[*curr].record_type;
 
 		if (!*eosaf && (rtype != R_RESTART) && (rtype != R_COMMENT)) {
 			/* Read the extra fields since it's not a special record */
 			read_file_stat_bunch(act, *curr, ifd, file_hdr.sa_act_nr, file_actlst,
-					     endian_mismatch, arch_64, file, file_magic);
+					     endian_mismatch, arch_64, file, file_magic, UEOF_STOP);
 		}
 
 		if ((lines >= rows) || !lines) {
@@ -987,7 +987,7 @@ void read_stats_from_file(char from_file[])
 	rows = get_win_height();
 
 	/* Read file headers and activity list */
-	check_file_actlst(&ifd, from_file, act, flags, &file_magic, &file_hdr,
+	check_file_actlst(&ifd, from_file, act, &file_magic, &file_hdr,
 			  &file_actlst, id_seq, FALSE, &endian_mismatch, &arch_64);
 
 	/* Perform required allocations */
@@ -1004,7 +1004,7 @@ void read_stats_from_file(char from_file[])
 		 */
 		do {
 			if (read_record_hdr(ifd, rec_hdr_tmp, &record_hdr[0], &file_hdr,
-					    arch_64, endian_mismatch, sizeof(rec_hdr_tmp))) {
+					    arch_64, endian_mismatch, UEOF_STOP, sizeof(rec_hdr_tmp))) {
 				/* End of sa data file */
 				return;
 			}
@@ -1023,7 +1023,7 @@ void read_stats_from_file(char from_file[])
 				 */
 				read_file_stat_bunch(act, 0, ifd, file_hdr.sa_act_nr,
 						     file_actlst, endian_mismatch, arch_64,
-						     from_file, &file_magic);
+						     from_file, &file_magic, UEOF_STOP);
 				if (sa_get_record_timestamp_struct(flags + S_F_LOCAL_TIME,
 								   &record_hdr[0],
 								   &rectime, NULL))
@@ -1096,13 +1096,13 @@ void read_stats_from_file(char from_file[])
 			do {
 				/* Read next record header */
 				eosaf = read_record_hdr(ifd, rec_hdr_tmp, &record_hdr[curr],
-							&file_hdr, arch_64, endian_mismatch, sizeof(rec_hdr_tmp));
+							&file_hdr, arch_64, endian_mismatch, UEOF_STOP, sizeof(rec_hdr_tmp));
 				rtype = record_hdr[curr].record_type;
 
 				if (!eosaf && (rtype != R_RESTART) && (rtype != R_COMMENT)) {
 					read_file_stat_bunch(act, curr, ifd, file_hdr.sa_act_nr,
 							     file_actlst, endian_mismatch, arch_64,
-							     from_file, &file_magic);
+							     from_file, &file_magic, UEOF_STOP);
 				}
 				else if (!eosaf && (rtype == R_COMMENT)) {
 					/* This was a COMMENT record: print it */
@@ -1468,7 +1468,7 @@ int main(int argc, char **argv)
 
 	/* 'sar' is equivalent to 'sar -f' */
 	if ((argc == 1) ||
-	    (((interval < 0) || INTERVAL_SET(flags)) && !from_file[0] && !to_file[0])) {
+	    ((interval < 0) && !from_file[0] && !to_file[0])) {
 		set_default_file(from_file, day_offset, -1);
 	}
 
