@@ -435,6 +435,12 @@ __print_funct_t xml_print_io_stats(struct activity *a, int curr, int tab,
 		sic->dk_drive_wblk < sip->dk_drive_wblk ? 0.0 :
 		S_VALUE(sip->dk_drive_wblk, sic->dk_drive_wblk, itv));
 
+	xprintf(tab, "<io-discard dtps=\"%.2f\" bdscd=\"%.2f\"/>",
+		sic->dk_drive_dio < sip->dk_drive_dio ? 0.0 :
+		S_VALUE(sip->dk_drive_dio, sic->dk_drive_dio, itv),
+		sic->dk_drive_dblk < sip->dk_drive_dblk ? 0.0 :
+		S_VALUE(sip->dk_drive_dblk, sic->dk_drive_dblk, itv));
+
 	xprintf(--tab, "</io>");
 }
 
@@ -728,29 +734,31 @@ __print_funct_t xml_print_disk_stats(struct activity *a, int curr, int tab,
 			"tps=\"%.2f\" "
 			"rd_sec=\"%.2f\" "
 			"wr_sec=\"%.2f\" "
+			"dc_sec=\"%.2f\" "
 			"rkB=\"%.2f\" "
 			"wkB=\"%.2f\" "
+			"dkB=\"%.2f\" "
 			"avgrq-sz=\"%.2f\" "
 			"areq-sz=\"%.2f\" "
 			"avgqu-sz=\"%.2f\" "
 			"aqu-sz=\"%.2f\" "
 			"await=\"%.2f\" "
-			"svctm=\"%.2f\" "
 			"util-percent=\"%.2f\"/>",
 			/* Confusion possible here between index and minor numbers */
 			dev_name,
 			S_VALUE(sdp->nr_ios, sdc->nr_ios, itv),
 			S_VALUE(sdp->rd_sect, sdc->rd_sect, itv), /* Unit = sectors (for backward compatibility) */
 			S_VALUE(sdp->wr_sect, sdc->wr_sect, itv),
+			S_VALUE(sdp->dc_sect, sdc->dc_sect, itv),
 			S_VALUE(sdp->rd_sect, sdc->rd_sect, itv) / 2,
 			S_VALUE(sdp->wr_sect, sdc->wr_sect, itv) / 2,
+			S_VALUE(sdp->dc_sect, sdc->dc_sect, itv) / 2,
 			/* See iostat for explanations */
 			xds.arqsz,	/* Unit = sectors (for backward compatibility) */
 			xds.arqsz / 2,
 			S_VALUE(sdp->rq_ticks, sdc->rq_ticks, itv) / 1000.0,	/* For backward compatibility */
 			S_VALUE(sdp->rq_ticks, sdc->rq_ticks, itv) / 1000.0,
 			xds.await,
-			xds.svctm,
 			xds.util / 10.0);
 	}
 
@@ -1924,11 +1932,17 @@ __print_funct_t xml_print_huge_stats(struct activity *a, int curr, int tab,
 	xprintf(tab, "<hugused>%llu</hugused>",
 		smc->tlhkb - smc->frhkb);
 
-	xprintf(tab--, "<hugused-percent>%.2f</hugused-percent>",
+	xprintf(tab, "<hugused-percent>%.2f</hugused-percent>",
 		smc->tlhkb ?
 		SP_VALUE(smc->frhkb, smc->tlhkb, smc->tlhkb) :
 		0.0);
 
+	xprintf(tab, "<hugrsvd>%llu</hugrsvd>",
+		smc->rsvdhkb);
+
+	xprintf(tab--, "<hugsurp>%llu</hugsurp>",
+		smc->surphkb);
+
 	xprintf(tab, "</hugepages>");
 }
 
@@ -2175,7 +2189,7 @@ __print_funct_t xml_print_fchost_stats(struct activity *a, int curr, int tab,
 			S_VALUE(sfcp->f_rxframes, sfcc->f_rxframes, itv),
 			S_VALUE(sfcp->f_txframes, sfcc->f_txframes, itv),
 			S_VALUE(sfcp->f_rxwords,  sfcc->f_rxwords,  itv),
-			S_VALUE(sfcp->f_txwords,  sfcc->f_txwords,  itv));
+			S_VALUE(sfcp->f_txwords,  sfcc->f_rxwords,  itv));
 	}
 	tab--;
 
